// libGnochon version 140204
//Cette version permet d'externaliser l'appel des fonction millis() et micros()
// Bonjour, voici ma petite librairie contenant une minuterie par milisecondes BlocTON(struct TON *pDATA)
// ,une par microsecondes BlocTON_Micros(struct TON_Micros *pDATA)
// et un dtecteur de front sur un booleen BlocFront(struct FRONT *pDATA).
// Voir plus bas pour les explications.
// Tout ceci est sous licence CC BY-NC-SA 4.0 http://creativecommons.org/licenses/by-nc-sa/4.0/
// pour me poser des questions, proposer des amliorations ou dire bonjour: gnochon@gmail.com

#include "Arduino.h"
#include "libGnochon.h"

  // Definition du type TON a utiliser avec BlocTON pour les minuteries en base de milisecondes.
  typedef struct TON{bool EN;bool DN;bool TT;bool PremierScan;unsigned long PRE;unsigned long ACC;unsigned long Millis;unsigned long MillisAncien;bool External;};
  // Definition du type TON_Micros a utiliser avec BlocTON_Micros pour les minuteries en base de microsecondes.
  typedef struct TON_Micros{bool EN;bool DN;bool TT;bool PremierScan;unsigned long PRE;unsigned long ACC;unsigned long Micros;unsigned long MicrosAncien;bool External;};
  // Definition du type FRONT a utiliser avec BlocFront.
  typedef struct FRONT{bool BIT; bool FrontM; bool FrontD; bool Avant;};

/*Ce bloc est une minuterie en millisecondes inspire du bloc TON des automates Allen Bradley. Voir le lien suivant pour plus d'information.
http://literature.rockwellautomation.com/idc/groups/literature/documents/rm/1785-rm001_-en-p.pdf
Pour fonctionner il faut crer un type comme ceci:
typedef struct TON{boolean EN;boolean DN;boolean TT;boolean PremierScan;unsigned long PRE;unsigned long ACC;unsigned long Millis;unsigned long MillisAncien;};
Puis crer pour chaque minuteries une variable de type TON comme ceci:
TON TON_A = {false,false,false,false,10000,0,0,0,false};
Il est bien important d'initialiser la variable avec ces valeurs. Les seules valeurs qui peut etre changes sont:
PRE qui reprsente le dlai en millisecondes
External qui dtermine si on utilise une valeur de temps externe.
Dans la structure les sous variables sont:
.EN -> Bit en criture qui active ou dsactive la minuterie. Utilisation un peu diffrente du TON A-B car dans celui-ci elle est en lecture seulement. Ici on l'utilise pour activer la minuterie non pas pour indiquer si elle est active.
.DN -> Bit en lecture qui indique si le dlai de la minuterie est coul.
.TT -> Bit en lecture qui indique que la minuterie est active mais que le dlai n'est pas coul.
.PremierScan -> Bit interne utilis pour indiquer la premire xcution du bloc aprs activation par .EN.
.PRE -> Entier en criture qui indique le dlai de la minuterie en millisecondes.
.ACC -> Entier en lecture qui indique le temps coul depuis l'activation de la minuterie par .EN.
.Millis -> Entier interne utilis pour mettre en mmoire la valeur de millis() au dbut d'une execution du bloc. C'est dans cette variable qu'on crit la valeur de temps externe lorsqu'utilis.
.MillisAncien -> Entier interne utiliser pour garder en mmoire la valeur de millis() au dbut de l'excution prcdente du bloc.
.External -> Booleen permettant de slectionner la source du temps entre la fonction micros()  l'interne ou une valeur de temps passe de l'externe. Vrai = externe.
*/
void BlocTON(struct TON *pDATA){
  if (pDATA->EN == true){		//Si la minuterie est active...
    if (pDATA->External == false){//la valeur des millisecondes au dbut de l'execution du bloc est mise en mmoire si le temps externe n'est pas activ
      pDATA->Millis = millis();
    }
    if (pDATA->ACC < pDATA->PRE){		// Si le temps coul est plus petite que le dlai  atteindre
      pDATA->DN = false;			// Le dlai n'est pas atteint
      pDATA->TT = true;			// Le dlai est en cours
        if (pDATA->PremierScan == true){	// Il sagit de la premire execution de ce bloc depuis sont activation (.EN == true)
          pDATA->PremierScan = false;	// Le drapeau indiquant la premire execution est mis  faux
        }
        else if (pDATA->MillisAncien > pDATA->Millis){// petite astuce anti-rebouclage pour les millisecondes. Aprs un certain temps le compteur de millisecondes de l'Arduino repart  0.
          pDATA->ACC = pDATA->ACC + pDATA->Millis + ((pow(2,32)-1) - pDATA->MillisAncien);// Le dlai dj coul+la valeur max des millisecondes de l'Arduino + la valeur des milisecondes au dbut de l'excution du bloc.
        }
        else {
          pDATA->ACC = pDATA->ACC + (pDATA->Millis - pDATA->MillisAncien);// En temps normal, le dlai dj coul plus la diffrence entre les millisecondes au dbut de cette excution et l'excution prcdente de ce bloc.
        }
    }
    else{// Une fois le dlai coul...
      pDATA->DN = true;	// Le dlai est termin
      pDATA->TT = false;	//La minuterie n'est plus entrein de compter
    }
  }
  else {//SI la minuterie n'est pas active...
    pDATA->DN = false;         //La minuterie n'est pas coule
    pDATA->TT = false;         //La minuterie n'est pas entrein de compter
    pDATA->ACC = 0;           //La minuterie n'a pas accumul de temps
    pDATA->PremierScan = true;//La minuterie en est a son premier scan
  }
  pDATA->MillisAncien = pDATA->Millis;
}

/* ** Ce bloc est une minuterie en microsecondes inspire du bloc TON des automates Allen Bradley. Voir le lien suivant pour plus d'information.
http://literature.rockwellautomation.com/idc/groups/literature/documents/rm/1785-rm001_-en-p.pdf
Cette minuterie prends en compte le rebouclage de la minuterie interne du processeur.
Il faut crer un type comme ceci:
typedef struct TON_Micros{bool EN;bool DN;bool TT;bool PremierScan;unsigned long PRE;unsigned long ACC;unsigned long Micros;unsigned long MicrosAncien;bool External; unsigned long Time;};
Puis crer pour chaque minuteries une variable de type TON_Micros comme ceci:
TON_Micros TON_A = {false,false,false,false,10000,0,0,0,false};
Il est bien important d'initialiser la variable avec ces valeurs. Les seules valeurs qui peut etre changes sont:
PRE qui reprsente le dlai en millisecondes
Time qui contient la valeur de temps externe.
Dans la structure les sous variables sont:
.EN -> Bit en criture qui active ou dsactive la minuterie. Utilisation un peu diffrente du TON A-B car dans celui-ci elle est en lecture seulement. Ici on l'utilise pour activer la minuterie non pas pour indiquer si elle est active.
.DN -> Bit en lecture qui indique si le dlai de la minuterie est coul. .DN est haut lorsque .ACC >= .PRE. .DN reste haut tant que .EN est haut.
.TT -> Bit en lecture qui indique que la minuterie est active mais que le dlai n'est pas coul.
.PremierScan -> Bit interne utilis pour indiquer la premire xcution du bloc aprs activation par .EN.
.PRE -> Entier long en lecture et criture qui contient le dlai de la minuterie en millisecondes.
.ACC -> Entier long en lecture qui cumule le temps coul depuis l'activation de la minuterie par .EN.
.Micros -> Entier long interne utilis pour mettre en mmoire la valeur de millis() au dbut d'une execution du bloc.
.MicrosAncien -> Entier long interne utilis pour garder en mmoire la valeur de millis() au dbut de l'excution prcdente du bloc.
.External -> Booleen permettant de slectionner la source du temps entre la fonction micros()  l'interne ou une valeur de temps passe de l'externe. Vrai = externe.
*/
void BlocTON_Micros(struct TON_Micros *pDATA){
  if (pDATA->EN == true){		//Si la minuterie est active...
    if (pDATA->External == false){//la valeur des millisecondes au dbut de l'execution du bloc est mise en mmoire si le temps externe n'est pas activ
      pDATA->Micros = micros();
    }
    if (pDATA->ACC < pDATA->PRE){	// Si le temps accumul est plus petite que le dlai de la minuterie...
      pDATA->DN = false;			// Le dlai n'est pas atteint
      pDATA->TT = true;			// L'accumulation du temps est en cours
        if (pDATA->PremierScan == true){	// S'il sagit de la premire execution de ce bloc depuis sont activation (.EN == true)
          pDATA->PremierScan = false;	// Le drapeau indiquant la premire execution est mis  faux
        }
        else if (pDATA->MicrosAncien > pDATA->Micros){// petite astuce anti-rebouclage pour les Microsecondes. Aprs un certain temps le compteur de Microsecondes de l'Arduino recommence  0.
          pDATA->ACC = pDATA->ACC + pDATA->Micros + ((pow(2,32)-1) - pDATA->MicrosAncien);// Le dlai dj coul+la valeur max des Microsecondes de l'Arduino + la valeur des milisecondes au dbut de l'excution du bloc.
        }
        else {
          pDATA->ACC = pDATA->ACC + (pDATA->Micros - pDATA->MicrosAncien);// En temps normal, le temps accumul = le temps dj accumul plus la diffrence entre les Microsecondes au dbut de cette excution et l'excution prcdente de ce bloc.
        }
    }
    else{             // Une fois le temps accumul >= au dlai...
      pDATA->DN = true; // Le dlai est termin
      pDATA->TT = false;// La minuterie n'est plus entrein d'accumuler le temps
    }
  }
  else {                    //SI la minuterie n'est pas active...
    pDATA->DN = false;        //La minuterie n'est pas coule
    pDATA->TT = false;        //La minuterie n'est pas entrein de compter
    pDATA->ACC = 0;           //La minuterie n'a pas accumul de temps
    pDATA->PremierScan = true;//La minuterie en est a son premier scan
  }
  pDATA->MicrosAncien = pDATA->Micros;//Mise en mmoire des microsecondes au dbut de cette excution pour comparer avec celles au dbut de la prochaine excution. 
}

/* Ce bloc est un deceteur de front. ** 
On lui donne une variable struct compos de quatre booleen.
.BIT La valeur  surveiller.
.FrontM se met vrai si la valeur de .BIT passe de faux  vrai. FrontM reste vrai pour une seule xcution du bloc.
.FrontD se met vrai si la valeur de .BIT passe de vrai  faux. FrontM reste vrai pour une seule xcution du bloc.
.Avant est utilis pour mettre en mmoire l'tat prcdent du .BIT.
Pour faire fonctionner ce bloc il faut crer un typedef comme ceci:
typedef struct FRONT{boolean BIT; boolean FrontM; boolean FrontD; boolean Avant;};// Dfinition du type FRONT utilis avec BlocFront. BlocFront permet d'indiquer un front montant ou descendant sur un boolen.
Il est prfrable d'initialiser toutes ces valeures  faux lors de la cration de la variable.
*/
void BlocFront(struct FRONT *pDATA){
  pDATA->FrontM = false;//Remise  faux des indicateurs de front car un front ne peut durer qu'une seule execution.
  pDATA->FrontD = false;//Remise  faux des indicateurs de front car un front ne peut durer qu'une seule execution.
  pDATA->FrontM = pDATA->BIT && (pDATA->BIT != pDATA->Avant);// Le front montant est dclar  condition que le bit soit haut et que l'tat prsent du bit ne soit pas le mme que lors du dernier scan.
  pDATA->FrontD = !pDATA->BIT && (pDATA->BIT != pDATA->Avant);// Le front descendant est dclar  condition que le bit soit bas et que l'tat prsent du bit ne soit pas le mme que lors du dernier scan.
  pDATA->Avant = pDATA->BIT;//Mise en mmoire de l'tat du bti pour comparer evac son tat lors de la prochaine excution.
}
